#ifndef _CRDBRELATION_H_
#define _CRDBRELATION_H_

#ifdef DO_RDB

// move this to generic-rel.h later...
#define COR_R_RDBREL (COR_R_OTHER + 50)

#include "BuiltinRelation.h"
#include "Rdb.h"
#include "Debug.h"

// This is a bit confusing:
// CRdbRelation is a part of the coral relation hierarchy,
// but RdbRelation is a part of the Rdb hierarchy.
// All CRdb... things are part of coral and talk to the rdb stuff

class Cor_CRdbRelation
: public Cor_Relation
{
  public:

    Cor_CRdbRelation(const Cor_CArg& newName, const RdbJoin& newJoinInfo);

    ~Cor_CRdbRelation();

    RdbRelation& RdbRel() const; // access the first RdbRelation
    
    //  override from Relation:
    
    virtual Cor_BindEnv *get_next(Cor_TupleIterator&); // defunct

    virtual Cor_Tuple *get_next_tuple(Cor_TupleIterator& iterator); 
				// get the next tuple using the iterator
    
    virtual void release(Cor_TupleIterator& iterator);
				// close the iterator
    
    virtual int insert_tuple(Cor_Tuple*); // inserts tuple in relation by 
				      // calling insert_new

    virtual int insert_new(Cor_ArgList& argList, Cor_BindEnv *pBindEnv) {
        return(insert_new(argList, pBindEnv, 0,0,0));
    }

    virtual int insert_new(Cor_ArgList& argList, Cor_BindEnv *pBindEnv,
			   Cor_BindEnv *env, Cor_Tuple* = 0, int = 0); 
				// check subsumption, then inserts 
				// into backend dbms.
				// the last three parameters are not used
    
    virtual tuple_delete(Cor_Tuple *tuple);	// delete all tuples matching 
					// the bindings of this tuple

    virtual void print_name(ostream *out); // print the name of the 
					 // relation on out

  private:
    
    const RdbJoin joinInfo;	// list of rdb relations to join
				// along with the join conditions

    RdbQueryVersions *queryVersions;

    RdbInsert *rdbInsert;

    RdbDeleteVersions *deleteVersions;
    

    RdbCursor *Init(Cor_TupleIterator& iterator);
				// initialize the based upon the iterator.
				// create a new cursor if the iterator is at
				// the top.

    RdbCursor *Bind(Cor_TupleIterator& iterator);
				// 

    Cor_Tuple *Done(Cor_TupleIterator& iterator);
				// 
				// always returns NULL

    int AsRdbItemList(Cor_ArgList& argList, Cor_BindEnv *pBindEnv,
		      RdbItemList& result, BitVector& adorn) const;
				// convert the argList (w/ pBindEnv)
				// to the result RdbItemList.
				// adorn[i] is set to true iff 
				// argList[i] is bound
				// returns: 0 on success
				//          1 on failure

    Cor_ArgList *AsArgList(RdbItemList& result);
				// convert RdbItemList to an argList
				// returns a "new" arglist which the caller
				// is responsible for freeing

    Cor_CRdbRelation(const Cor_CRdbRelation&); // not allowed

    Cor_CRdbRelation& operator =(const Cor_CRdbRelation&); // not allowed
};

/*****************************************************************************/

// These two classes are used to keep a list of coral variable names &
// their position in the arglist.  Then the list can be sorted by name
// and then you can easily find the joins between the relations.

class Cor_CRdbVar {
  public:
    int rel;
    int col;
    Cor_CArg name;		// variable name in arglist
    
    Cor_CRdbVar(int newRel, int newCol, Cor_CArg& newName)
      { rel = newRel; col = newCol; name = newName; }
};

class Cor_CRdbVarPtrList
: public DList<Cor_CRdbVar>
{
  public:
    void SortByName();
};


/***************************************************************************/

// Global functions:
int CreateCRdbRelation(Cor_CArg& newName, RdbJoin& join);
int CreateCRdbJoinRelation(Cor_CArg *pfaNewRel, Cor_BindEnv *pBindEnv);
void CreateCRdbJoinList(RdbJoinCondList& joinList, Cor_CRdbVarPtrList& varList);

/***************************************************************************/

inline RdbRelation& Cor_CRdbRelation::RdbRel() const
{
    return joinInfo.Rel(0); 
}

/***************************************************************************/

inline Cor_BindEnv *Cor_CRdbRelation::get_next(Cor_TupleIterator&) // defunct
{
    ASSERT(0);
    return NULL;
}

/***************************************************************************/
/***************************************************************************/

#endif // DO_RDB

#endif // _CRDBRELATION_H_
