// *************************************************************************
//    DebugSolvers.h
// *************************************************************************

#ifndef _DebugSolvers_H_
#define _DebugSolvers_H_

// *************************************************************************

// this class is used for  StartTimeSolver(), EndTimeSolver,CputimeSolver
// because getrusage is not available on HP.

#include <unistd.h>
#include <sys/times.h>

#include "CoralError.h"

class Cor_CcoralTimer {

      // coral_start does not get reset!
    struct tms   start, end, coral_start;
    clock_t      wall_start;		      
    int          clktck;

  public:
        // coral_usr is the accumulative usr time
    double       wall, sys, usr, coral_usr;

    Cor_CcoralTimer() {
        if ((clktck = sysconf(_SC_CLK_TCK)) < 0) {
            CORAL_error(COR_FATAL_ERROR, "","Coral timer: can't call sysconf");
            clktck = 100;
        }
        times(&coral_start);
        reset();
    }

   ~Cor_CcoralTimer() {};

    void reset() { wall_start = times(&start); }

    void look() {
        wall = (times(&end) - wall_start) / double (clktck);
        sys = (end.tms_stime - start.tms_stime + end.tms_cstime 
    	    - start.tms_cstime) / double (clktck);
        usr = (end.tms_utime - start.tms_utime + end.tms_cutime 
    	    - start.tms_cutime) / double (clktck);
        coral_usr = (end.tms_utime - coral_start.tms_utime + end.tms_cutime 
	        - coral_start.tms_cutime) / double (clktck);
    }
};

class Cor_BuiltinRelation;
class Cor_TupleIterator;
class Cor_BindEnv;
class Cor_Tuple;

// *************************************************************************

Cor_BindEnv *Cor_ResetCounterSolver(Cor_BuiltinRelation&,
                Cor_TupleIterator& iterator);
Cor_BindEnv *Cor_CounterSolver(Cor_BuiltinRelation&,
                Cor_TupleIterator& iterator);

Cor_BindEnv *Cor_DebugSolver(Cor_BuiltinRelation& rel,
                Cor_TupleIterator& iterator);
Cor_BindEnv *Cor_StartTimeSolver(Cor_BuiltinRelation&,
                Cor_TupleIterator& iterator);
Cor_BindEnv *Cor_EndTimeSolver(Cor_BuiltinRelation&,
                Cor_TupleIterator& iterator);
Cor_Tuple   *Cor_CputimeSolver(Cor_Tuple *query);


Cor_BindEnv *Cor_BreakSolver(Cor_BuiltinRelation&, Cor_TupleIterator& iterator);
Cor_BindEnv *Cor_ClearBreakSolver(Cor_BuiltinRelation&,
                Cor_TupleIterator& iterator);
Cor_BindEnv *Cor_PrintModuleSolver(Cor_BuiltinRelation&,
                Cor_TupleIterator& iterator);

Cor_BindEnv *Cor_ShowMemSolver(Cor_BuiltinRelation&,
                Cor_TupleIterator& iterator);


// *************************************************************************

#endif _DebugSolvers_H_
