// *************************************************************************
//    GoalNode.h
// *************************************************************************

#ifndef _GoalNode_H_
#define _GoalNode_H_

#include "Tuple.h"
#include "ArgList.h"
#include "Entity.h"
#include "Debug.h"

// *************************************************************************

class Cor_ContextNode;
class Cor_StorageRelation;

// *************************************************************************

class Cor_GoalNode : public Cor_Tuple {

  public:
    Cor_GoalNode        *parent_goal;
    Cor_ContextNode     *c_node;
    Cor_StorageRelation *owner_rel;
    // the above fields represent the semantics of goal nodes in the set

    char mark;
    char negated;
    // used to check if program has cyclic negation/aggregation 

    Cor_GoalNode *parent_in_set;
    int rank_in_set;
    // the above two fields are for tarjan's union-find algorithm

/*    // Messes up cause we are derived from Tuple 
#ifdef    REUSE_MEMORY
    void* operator new(size_t size) {
        return(Coral_Free->acquire(size));
    }

    void  operator delete(void *trash, size_t size) {
        Coral_Free->dispose((Cor_Ent_Element*) trash, size);
    }
#endif
*/

    Cor_GoalNode(Cor_StorageRelation *rel, int neg = 0);
    Cor_GoalNode(int arity, Cor_StorageRelation *rel, int neg = 0);
    Cor_GoalNode(Cor_ArgList *arglist, Cor_BindEnv *env,
            Cor_StorageRelation *rel, int neg = 0);
    Cor_GoalNode(Cor_ArgList&a, Cor_StorageRelation *rel, int neg = 0);
    Cor_GoalNode(Cor_ArgList *arglist, Cor_StorageRelation *rel, int neg = 0);

   ~Cor_GoalNode(); 

    void set_defaults(Cor_StorageRelation *rel, int neg);

    inline void create_goalnode_info(Cor_GoalNode *par) { parent_goal = par; } 

    Cor_GoalNode *find_canonical_elem();
    Cor_ContextNode *find_context_node();
    Cor_GoalNode *link_goals(Cor_GoalNode *g);

    inline int is_marked()        { return mark; }
    inline void mark_goal()        { mark = 1; }

    int is_magic_goal();
    void print_all(ostream *file);
    Cor_StorageRelation *find_done_relation();
};

// *************************************************************************

#endif _GoalNode_H_
