#ifndef _RDBCOMPILABLEDML_H_
#define _RDBCOMPILABLEDML_H_

// Any database statements that can be "prepared" and kept around for
// a while should be derived from this class.

// The PrepareFn eliminates the need to make this a virtual base class
// for the sole purpose of defining one virtual function

#include "RdbError.h"
#include "RdbDatabase.h"
#include "String.h"
#include "Boolean.h"


class RdbCompilableDML 
: public RdbError
{
  public:

    RdbCompilableDML(RdbDatabase& newDb); // establish which database to use

    virtual ~RdbCompilableDML();

    void Invalidate();		// make the DML as not ready

    Boolean IsReady() const;	// is the DML prepared in the backend?

    RdbDatabase& Database() const; // which database is this for?

    int Prepare();		// prepare this dml

    String DatabaseName() const; // database name for this dml

    String SystemName() const;	// system name for this dml

  protected:

    String statement;		// sql statement 

    String id;			// statement identifier to dbms

    int ready;			// is this statement prepared to execute?

    RdbDatabase& database;	// database that the dml is intended for

    static const String questionMark;
				// used because default parm 'String("?")'
				// was not working in g++ 2.6.0

  private:

    RdbCompilableDML(const RdbCompilableDML& other); // N/A

    RdbCompilableDML& operator =(const RdbCompilableDML& other); // N/A
};


#include "RdbCompilableDML.i"

#endif // _RDBCOMPILABLEDML_H_
