#ifndef _RDBMANAGER_H_
#define _RDBMANAGER_H_

// This is the top level manager of all the rdb systems.
// This is the only base Rdb module that references the derived types.

// Its purpose is to allow users of the system to avoid placing
// switches in their code to send messages to the right subclass, e.g.,
// to instantiate objects.  Some of this is supported via virtual functions,
// but sometimes that is more difficult than it's worth.

#include "RdbError.h"
#include "String.h"


class RdbSystem;		// forward declaration


class RdbManager
: public RdbError
{
  public:

    enum SystemType {
	INGRES,
	SYBASE,
	INFORMIX,
      };

    RdbManager();

    ~RdbManager();

    RdbSystem* LookupSystem(const String _systemName) const;
				// find the system by name (case insensitive)
				// return null if not found

    RdbSystem* LookupSystem(const SystemType _systemType) const;
				// find the system by type
				// return null if not found

    ErrorCode Commit();		// commit to all systems/databases
				// this could result in a rollback if there
				// is some error.

    ErrorCode Rollback();	// abort to all systems/databases

  private:

    enum { SYSTEM_COUNT = 3 };	// count of system types (must match!!)

    RdbSystem* system[SYSTEM_COUNT]; // one pointer for each system

    static const char* systemName[SYSTEM_COUNT];
				// textual names of the systems
};


#endif // _RDBMANAGER_H_
