#ifndef _RDBSYBASEDATABASE_H_
#define _RDBSYBASEDATABASE_H_

#include "RdbDatabase.h"
#include "RdbSybaseProcess.h"
#include "String.h"

// Forward Declarations:
class RdbSybaseSystem;


class RdbSybaseDatabase
: public RdbDatabase
{
  public:
    
    RdbSybaseDatabase(RdbSybaseSystem& sybaseSystem,
		      const String newDbName,
		      const String userid = String(),
		      const String password = String(),
		      const String serverName = String());
				// open sybase database called newDbName
				// in the server serverName (or default)
				// with the userid & plain text password.

    ~RdbSybaseDatabase();

    virtual RdbQuery* NewQuery(RdbJoin& joinInfo, const BitVector& adorn);
				// allocate a query on the joinInfo

    virtual RdbInsert* NewInsert(RdbRelation& rel);
				// allocate an insert statment on rel

    virtual RdbDelete* NewDelete(RdbRelation& rel, const BitVector& adorn);
				// allocate a delete statment on rel

    RdbSybaseSystem& System() const; // return the system for the database

    virtual void const* Implementor() const;
				// virtual access to 'this'

  private:

    RdbSybaseProcessPool* processPool;

    RdbSybaseSystem& system;

    // see RdbDatabase:

    virtual ErrorCode VConnect();

    virtual ErrorCode VDisconnect();

    virtual ErrorCode VImmediateSql(const String& sql);

    virtual ErrorCode VCommit();

    virtual ErrorCode VRollback();

    virtual ErrorCode VPrepare(const String& id, const String& statement);

    virtual String VGetErrorMessage();

    virtual ErrorCode VSetFocus();

    virtual ErrorCode VRemoveFocus();

    RdbSybaseDatabase(const RdbSybaseDatabase&); // na

    RdbSybaseDatabase& operator =(const RdbSybaseDatabase&); // na

    friend class RdbSybaseProcess; // for access to the process pool
};


#include "RdbSybaseDatabase.i"

#endif // _RDBSYBASEDATABASE_H_
