// *************************************************************************
//    BuiltinBinaryRelation.C
// *************************************************************************

#include "BuiltinBinaryRelation.h"

#include "InitBuiltins.h"
#include "Globals.h"
#include "TupleIterator.h"
#include "CoralError.h"
#include "Debug.h"

// *************************************************************************
Cor_BuiltinBinaryRelation::Cor_BuiltinBinaryRelation(int a, Cor_CArg *n,
        BinaryProc bin_op) : Cor_BuiltinRelation(a, n, NULL)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    op = bin_op;
}

/*****************************************************************************/
Cor_BindEnv *Cor_BuiltinBinaryRelation::get_next(Cor_TupleIterator& iterator)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    Cor_BindEnv *env = iterator.bindenv;

    if (iterator.ipos > 0) {
        iterator.stack_mark.pop_to();
        iterator.set_no_match(); 
        return env;
    }
    iterator.ipos++;
    iterator.stack_mark.get_mark();

    // Term terms[3];
    for (int i = 0; i < 3; i++) {
        Cor_static_terms[i].expr = iterator.arg_list[i],
            Cor_static_terms[i].bindenv = env;
        Cor_full_dereference_term(Cor_static_terms[i]);
    }
    
#    ifdef DO_TRACE
        if (Cor_exEnv->dbg_get_next) {
            *(Cor_exEnv->trace_strm) << "BuiltinRelation:";
            Cor_static_terms[2].printon(Cor_exEnv->trace_strm);
            *(Cor_exEnv->trace_strm) << " = " <<  name.symbol() << " (";
            Cor_static_terms[0].printon(Cor_exEnv->trace_strm);
            *(Cor_exEnv->trace_strm) << ",";
            Cor_static_terms[1].printon(Cor_exEnv->trace_strm);
            *(Cor_exEnv->trace_strm) << ")\n";
        }
#    endif

    // Specialized for numeric binary operations
    // WARNING: These operators work only one way currently. There should
    // be different operators called depending on the binding pattern.

    if (!(Cor_static_terms[0]).expr.is_num()
        || !(Cor_static_terms[1]).expr.is_num() )
    {
        CORAL_error(COR_NON_NUMERIC_ARGS, NULL, "get_next");
        *(Cor_exEnv->err_strm) << "(";
        Cor_static_terms[0].printon(Cor_exEnv->err_strm);
        *(Cor_exEnv->err_strm) << ",";
        Cor_static_terms[1].printon(Cor_exEnv->err_strm);
        *(Cor_exEnv->err_strm) << ")\n";
        iterator.set_no_match();
        iterator.stack_mark.pop_to();
        return(env);
    }

    //Term result_term;
    (*op)(Cor_usable_terms[0].expr, Cor_static_terms[0].expr,
	  Cor_static_terms[1].expr);
    Cor_usable_terms[0].bindenv = NULL;
    if (Cor_usable_terms[0].expr.is_empty()) {
        iterator.set_no_match();
        iterator.stack_mark.pop_to();
        return(env);
    }
    
    if (!Cor_unify_args (Cor_usable_terms[0], Cor_static_terms[2])) {
        iterator.set_no_match();
        iterator.stack_mark.pop_to();
        return(env);
    }

#    ifdef DO_TRACE
        if (Cor_exEnv->dbg_get_next) {
            *(Cor_exEnv->trace_strm) << "Result = ";
            Cor_static_terms[2].printon(Cor_exEnv->trace_strm);
            *(Cor_exEnv->trace_strm) << endl;
        }
#    endif

    return(env);
}

/*****************************************************************************/

