//********************************************
// NLExtJoin.C
//
// Ravi Murthy
//********************************************

#ifdef    WITH_PERSISTENCE
#include <assert.h>
#include "Plan.h"
#include "ExtJoin.h"
#include "NLExtJoin.h"

//**********************************************************

// This is the constructor for an internal join node.
Cor_NLExtJoin::Cor_NLExtJoin(Cor_TupleIterator* left, Cor_TupleIterator *right)
    :Cor_ExtJoin(left, right)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    reset_second = 1;
}

// This is the constructor for a leaf join node.
Cor_NLExtJoin::Cor_NLExtJoin(Cor_ExtJoin* left, Cor_TupleIterator *right)
    :Cor_ExtJoin(left, right)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    reset_second = 1;
}

Cor_NLExtJoin::~Cor_NLExtJoin()
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif

}

Cor_Tuple *Cor_NLExtJoin::get_next_tuple(Cor_TupleIterator& it)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    Cor_Tuple *outertuple, *innertuple;

    if (leaf == 1) {
        while (1) {
            if (reset_second == 1) {
                assert(left_rel != NULL);
                outertuple = left_rel->get_next_tuple();
                if (left_rel->no_match()) {
                    it.set_no_match();
                    return NULL;
                }
                assert(right_rel != NULL);
                right_rel->reset();
                reset_second = 0;
            }
    
            assert(right_rel != NULL);
            innertuple = right_rel->get_next_tuple();
            if (right_rel->no_match()) {
                reset_second = 1;
            } else {
                it.reset_no_match();
                return NULL;
            }
        }
    }


    // it is not a leaf - means it is an internal node.

    while (1) {
        if (reset_second == 1) {
            assert(ext_left_rel != NULL);
            outertuple = ext_left_rel->get_next_tuple(it);
            if (it.no_match()) {
                return NULL;
            }
            assert(right_rel != NULL);
            right_rel->reset();
            reset_second = 0;
        }
    
        assert(right_rel != NULL);
        innertuple = right_rel->get_next_tuple();
        if (right_rel->no_match()) {
            reset_second = 1;
        } else {
            it.reset_no_match();
            return NULL;
        }
    }
}

// *************************************************************************
#endif // WITH_PERSISTENCE
