/*****************************************************************************/
//    CoreHashTable.C
/*****************************************************************************/

#include <stdio.h>
#include <string.h>

#include "CoreHashTable.h"
#include "Assert.h"

/*****************************************************************************/
CoreHashTable::CoreHashTable(EqualsFn eqf1, int initsize1, double incr_ratio1,
    double max_occupancy1)
    : corGenericHashTable(eqf1, initsize1, incr_ratio1, max_occupancy1)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    table = new corHashEntry[max_size];
    // bzero(table, max_size *sizeof(HashEntry));
    memset(table, 0, max_size *sizeof(corHashEntry));
}

/*****************************************************************************/
/*
    The insert function returns the argument entry if entry did not
    already exist, else it returns the address of the existing 
    element that equals entry (under equalsfn). If to_delete is set
    and the input entry is already in the table, the input entry
    is deleted to recover the storage allotted to it. to_delete
    is a var parameter that on return specifies whether a delete
    occured within this function
*/

void* CoreHashTable::insert(void *entry, int hashval, int& to_delete)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    // PRAVEEN:: Come back to this !!

    int bucket = hashval % max_size;
    
    ASSERT (entry != NULL);

    if (table[bucket].data == NULL) {
        table[bucket].data = entry;
        table[bucket].hashval = hashval;
    }
    else {
        corHashEntry *temp_entry = table + bucket;
        while (temp_entry) {
            if ((temp_entry->hashval == hashval) &&
                ((*equalsfn)(temp_entry->data, entry)))
            {
                if (to_delete) delete entry;
                return(temp_entry->data);
            }
            temp_entry = temp_entry->next;
        }
        temp_entry = new corHashEntry;
        temp_entry->data = table[bucket].data;
        temp_entry->hashval = table[bucket].hashval;
        temp_entry->next = table[bucket].next;
        table[bucket].next = temp_entry;
        table[bucket].data = entry;
        table[bucket].hashval = hashval;
    }

    cur_size++;
    rehash_if_needed();
    to_delete = 0;            // Specify that no delete has occured
    return entry;
}

/*****************************************************************************/
void* CoreHashTable::lookup(void *entry, int hashv)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    int bucket = hashv % max_size;

    if (table[bucket].data == NULL) return NULL;

    corHashEntry *temp_entry = table + bucket;
    while (temp_entry) {
        if ((temp_entry->hashval == hashv) &&
            ((*equalsfn)(temp_entry->data, entry)))
                return(temp_entry->data);

        temp_entry = temp_entry->next;
    }

    return NULL;
}

/*****************************************************************************/
void CoreHashTable::rehash_bigger()
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    int old_max_size = max_size;
    int i;
    
    max_size = int(max_size * incr_ratio);
    limit = int(max_size * max_occupancy);
    cur_size = 0;
    
    corHashEntry *old_table = table;
    corHashEntry *temp_entry, *temp1_entry;

    table = new corHashEntry[max_size];
    memset(table, 0, max_size *sizeof(corHashEntry));
    for (i=0; i< old_max_size; i++) {
        if (!old_table[i].data) continue;
        temp_entry = old_table + i;
        while (temp_entry) {
            insert(temp_entry->data, temp_entry->hashval);
            temp1_entry = temp_entry->next;
            if (temp_entry != old_table +i) delete temp_entry;
            temp_entry = temp1_entry;
        }
        // there is a memory leak here, as the Hash_Entry structs in the
        // list are not reclaimed.  Should add code to do this at some point
        // -- PRAVEEN
    }
    delete old_table;
}

/*****************************************************************************/

