// *************************************************************************
//    GenericIndex.h
// *************************************************************************

#ifndef _GenericIndex_H_
#define _GenericIndex_H_

#include "ArgList.h"
#include "StorageRelation.h"
#include "BindEnv.h"
#include "BitVector.h"
#include "Tuple.h"
#include "TupleIterator.h"

class Cor_GenericIndexSet; 

// *************************************************************************

class Cor_GenericIndex {

  friend class Cor_GenericIndexSet;

  protected:
    Cor_StorageRelation *relation;        // relation that is being indexed

      // for argument form indexing only next field is relevant
    Cor_BitVector                 *argset; 

      // for pattern form indexing only next three fields are relevant
    Cor_ArgList     *pattern;         // pattern to be used for indexing
    int              num_vars_pat;    // number of variables in pattern
    Cor_ArgList     *var_names;       // subset of variables in pattern
                                      // that are used for indexing
    
  public: 
    Cor_GenericIndex(Cor_StorageRelation *rel, Cor_BitVector *aset) {
        relation = rel;
        argset = aset->dup();
        pattern = NULL;
        num_vars_pat = 0;
        var_names = NULL;
     }

    Cor_GenericIndex(Cor_StorageRelation *rel, Cor_ArgList *pat,
            int n_var, Cor_ArgList *v_nm) {
        relation = rel; 
        pattern = pat;
        num_vars_pat = n_var;
        var_names = v_nm;
        argset = NULL;
     }
    /*
     *    this assumes that ArgList *pat and ArgList *v_nm
     *    have been created with the same variable numbers assigned 
     *    to the same variable names, and that the variable numbers are
     *    sequentially assigned from 0, with no number being skipped 
     *    in between.
     */
    // NEW_CHNG:wX
    // required for this abstract class
    virtual ~Cor_GenericIndex() {
        delete argset;
    }

    inline int add_index_to_reln(Cor_StorageRelation *reln) {
        if (argset == NULL) { // add pattern form index
            return reln->add_index(pattern, num_vars_pat, var_names);
        } else {
              // add argument form index
            return reln->add_index(argset);
        }
     }

    virtual void initialize_iterator( Cor_TupleIterator& ) = 0;

    inline int is_argument_form()    { return(argset != NULL); }

    virtual Cor_Tuple *get_next_tuple(Cor_TupleIterator& ) = 0;
    virtual Cor_BindEnv *get_next( Cor_TupleIterator& ) = 0;
    int matches(Cor_GenericIndex *ind);
    virtual int index_type() = 0;
    virtual int is_subsumed(Cor_Tuple *tuple) = 0;
    
    virtual int  insert_tuple( Cor_Tuple *tuple );
    virtual int  delete_tuple( Cor_Tuple *tuple );
    virtual void delete_all_tuples(int force_del=0);
    virtual void move_all_entries(Cor_GenericIndex *ind) = 0;

      // Empty the index, but do NOT delete tuples (these are shared)
      // Retain the index, for reuse, and reset its size
    virtual void reset_index() = 0;

      // For debugging
    virtual void show_all_tuples(ostream& out);
};

// *************************************************************************

enum IndexType { COR_HASH_INDEX };

// *************************************************************************

char *Cor_simplify_index(Cor_ArgList *pat, int n_var, Cor_ArgList *v_nm);

// *************************************************************************

#endif _GenericIndex_H_
