
%{
#include "Scanner.h"
#include "Annotations.h"
#include "Literal.h"
#include "Interface.h"
#include "ParseActions.h"
#include "CoralError.h"
#include "Rule.h"
#include "Collection.h"

#include "ParseUtils.h"
#include "BuiltinSymbols.h"
#include "ArgList.h"
#include "CArg.h"
#include "ClientProtocol.h"

void yyerror(char *s);

EXTERN int yylex(YYSTYPE *lvalp);
EXTERN int yyparse();

static char *fname = "yyparse";

    // Redefine the parser to use this "lexer" so that we can switch
    // scanners (aka lexers) depending who is one the top of the stack.
    // The function will actually be called as:
    //     (parseEnv->lexer)(&yylval)
    // assuming YYPURE is defined and YYLSP_NEEDED is not.
    // This is done here to get around bison.simple's definition of YYLEX.
#undef    yylex
#define   yylex    (Cor_parseEnv->lexer)

Cor_SymTable *Cor_BuiltinRelTable;

%}

// This next line makes the parser re-entrant.
%pure_parser

%type   <pred>    pred clause_lhs clause_lhs_list clause_lhs_element
%type   <carg>    arg arg_fact arg_primary var_arg simple_arg
%type   <carg>    id anno_name adorn_string opt_adorn
%type   <carg>    symbol relop
%type   <ca_list> arglist opt_arglist grouping_list
%type   <carg>    member_list
%type   <integer> plus_or_minus

// If a token is added here, it should be added to Gram.y & CArg.y as well.
// The parsers must have the same tokens because they are using the
// same lexers.
%token  CONS NIL
%token  APPEND UPDATE ASSIGN DELETE
%token  MARK LPAREN RPAREN COMMA COLON SLASHSLASH PERIOD LBRACE RBRACE ARROW
%token  SIPS LBRACKET RBRACKET UNDERSCORE VBAR PERCENT NOT QUESTION DOLLAR TRUE
%token  SUM PROD COUNT CMIN CMAX AVG ANY DISTINCT COUNT_D SUM_D PROD_D AVG_D
%token  DEL START_C_MODULE END_C_MODULE MODULE EXPORT ALIAS END_MODULE
%token  AGG_SELECTOR AGG_SELECTOR_MANY_TO_ONE AGG_SELECTOR_MANY_TO_MANY
%token  PLUS MINUS STAR SLASH EQUAL NOTEQUAL LESS GREATER LESSEQUAL GREATEREQUAL
%token  LSHIFT RSHIFT BITXOR BITAND BITNOT BITOR MOD
%token  AT CUT DOUB_AT
%token  <carg> FSYMBOL SYMBOL VAR OP RELOP
%token  <carg> NUM
%left   BITOR
%left   BITXOR
%left   BITAND
%left   LSHIFT RSHIFT
%left   PLUS MINUS
%left   STAR SLASH SLASHSLASH MOD
%right  BITNOT

%%
unit:   clause

    |   MODULE id PERIOD
            clause_list
        END_MODULE PERIOD

    |   START_C_MODULE PERIOD
            clause_list
        END_C_MODULE PERIOD

    |   START_C_MODULE PERIOD
            MODULE id PERIOD
                clause_list
            END_MODULE PERIOD
        END_C_MODULE PERIOD
;


clause_list:    clause_list clause
           |    /* Nothing */
;

/* export pred1(adorn11, adorn12, ...), pred2(adorn21, ..), ... */
export_clause:    EXPORT arg_fact

    |    export_clause COMMA arg_fact

    |    EXPORT symbol LBRACKET opt_arglist RBRACKET

    |    export_clause COMMA symbol LBRACKET opt_arglist RBRACKET
;

annotation: 
         AT symbol plus_or_minus

    |    AT symbol symbol opt_adorn plus_or_minus

    |    AT symbol symbol opt_adorn
                    LPAREN opt_arglist RPAREN
                    LPAREN opt_arglist RPAREN plus_or_minus

    |    AT symbol symbol opt_adorn
                    LPAREN opt_arglist RPAREN plus_or_minus

    |    AT symbol symbol opt_adorn
                    LPAREN opt_arglist RPAREN
                    LPAREN opt_arglist RPAREN
                    simple_arg plus_or_minus
;

clause:  clause_lhs PERIOD

    |    clause_lhs MARK   rhs_pred_list PERIOD

    |    clause_lhs APPEND rhs_pred_list PERIOD

    |    clause_lhs DELETE rhs_pred_list PERIOD

    |    clause_lhs UPDATE rhs_pred_list PERIOD

    |    clause_lhs ASSIGN rhs_pred_list PERIOD

    |    QUESTION rhs_pred_list PERIOD

    |    export_clause PERIOD

    |    annotation PERIOD

    |    PERIOD

    |    error PERIOD clause
;

plus_or_minus:
              PLUS      { $$ = 1;}
        |     MINUS     { $$ = 0; }
        |   /* None */  { $$ = 1;}
;

clause_lhs:  id COLON clause_lhs
        |    id MINUS COLON clause_lhs
        |    clause_lhs_list
;

clause_lhs_list: clause_lhs_list COMMA clause_lhs_element
        |        clause_lhs_element
;

clause_lhs_element: symbol LPAREN opt_arglist RPAREN
        |           DEL symbol LPAREN opt_arglist RPAREN
        |           symbol
;

rhs_pred_list:   rhs_pred_list COMMA rhs_pred
        |        rhs_pred
;

rhs_pred:    pred
        |    /* Nothing */
;

pred:       symbol LPAREN opt_arglist RPAREN
        |   symbol

        |   arg relop arg

        |   NOT arg relop arg

        |   NOT symbol LPAREN opt_arglist RPAREN

        |   NOT symbol

        |   CUT
;

relop:   EQUAL         { $$ = Cor_EquOpSymbol; }
    |    NOTEQUAL      { $$ = Cor_NeqOpSymbol; }
    |    LESS          { $$ = Cor_LssOpSymbol; }
    |    GREATER       { $$ = Cor_GrtOpSymbol; }
    |    LESSEQUAL     { $$ = Cor_LeqOpSymbol; }
    |    GREATEREQUAL  { $$ = Cor_GeqOpSymbol; }
;

arglist:     arglist COMMA arg
        {
            ($1)->push($3);
        }

        |    arg
        {
            $$ = new Cor_CArgStack();
            ($$)->push($1);
        }
;

grouping_list: arg
        {
            $$ = new Cor_CArgStack();
            ($$)->push($1);
        }
;

opt_arglist:    arglist
    |   /* Nothing */
        {
              // Actually allocate one so everyone can see its empty
            $$ = new Cor_CArgStack();
        }
;

arg:    arg_primary
    |   arg PLUS arg
        {
            $$ = new Cor_CArg(ADD_OP_ARG,$1,$3);
        }

    |   arg MINUS arg
        {
            $$ = new Cor_CArg(SUB_OP_ARG,$1,$3);
        }

    |   arg STAR arg
        {
            $$ = new Cor_CArg(MULT_OP_ARG,$1,$3);
        }

    |   arg SLASH arg
        {
            $$ = new Cor_CArg(DIV_OP_ARG,$1,$3);
        }

    |   arg SLASHSLASH arg
        {
            $$ = new Cor_CArg(IDIV_OP_ARG,$1,$3);
        }

    |   arg LSHIFT arg
        {
            $$ = new Cor_CArg(LSHFT_OP_ARG,$1,$3);
        }

    |   arg RSHIFT arg
        {
            $$ = new Cor_CArg(RSHFT_OP_ARG,$1,$3);
        }

    |   arg BITXOR arg
        {
            $$ = new Cor_CArg(BT_XOR_OP_ARG,$1,$3);
        }

    |   arg BITAND arg
        {
            $$ = new Cor_CArg(BT_AND_OP_ARG,$1,$3);
        }

    |   arg BITOR arg
        {
            $$ = new Cor_CArg(BT_OR_OP_ARG,$1,$3);
        }

    |   BITNOT arg_primary
        {
            $$ = new Cor_CArg(BT_NOT_OP_ARG,$2,NULL);
        }

    |   arg MOD arg
        {
            $$ = new Cor_CArg(MOD_OP_ARG,$1,$3);
        }

    |   MINUS arg_primary    %prec BITNOT
        {
            if (($2)->is_num()) {
                if (($2)->is_int())
                    $$ = new Cor_CArg(-(($2)->NumIval()));
                else
                    $$ = new Cor_CArg(-(($2)->NumDval()));
                $2->destroy();
            } else {
                $$ = new Cor_CArg(SUB_OP_ARG,$2,NULL);
            }
        }
;

arg_primary:    arg_fact
           |    LPAREN arg RPAREN
           {
               $$ = $2;
           }

simple_arg:   symbol LPAREN opt_arglist RPAREN
        {
            $$ = Cor_create_functor($3, $1, Cor_parserStruct);
            $1->destroy();
            delete $3;
        }

arg_fact:    symbol
    |   NUM
    |   var_arg
    |   UNDERSCORE
        {
            $$ = new Cor_CArg(DONT_CARE_ARG);
        }

    |   DOLLAR
        {
            $$ = new Cor_CArg(PARAMETER_ARG);
        }

    |   simple_arg
    |   LBRACKET arglist VBAR arg RBRACKET
        {
            $$  = Cor_create_list($2, $4);
            delete $2;
        }

    |   LBRACKET opt_arglist RBRACKET
        {
            $$ = Cor_create_list($2, NULL);
            delete $2;
        }

    |   CONS LPAREN arg COMMA arg RPAREN
        {
            $$ = new Cor_CArg($3,$5);
            //$3->destroy();
            //$5->destroy();
        }

    |   NIL
        {
            $$ = new Cor_CArg(NIL_ARG);
        }

    |   LESS grouping_list GREATER
        {

              // Temporially disabled, until find spot to set agg_flag
            if (1 || Cor_parserStruct->lhs_flag || Cor_parserStruct->agg_flag) {
                Cor_parseEnv->complexRelation++;
                if (Cor_exEnv->dbg_aggregates)
                    *(Cor_exEnv->err_strm)
                        << "nested att found complex relation "
                        << Cor_parseEnv->complexRelation << endl;
                $$ = Cor_create_multi_grouping(Cor_parserStruct,$2);
            } else {
                CORAL_error(COR_GROUPING_ON_RHS, NULL, fname, __FILE__,
                        __LINE__);
                YYERROR;
            }
            delete $2;
        }

    |   LBRACE member_list RBRACE
        {

            $$ = $2;

            Cor_parseEnv->complexRelation++;

            if (Cor_exEnv->dbg_aggregates)
                *(Cor_exEnv->err_strm)
                    << "nested att found complex relation "
                    << Cor_parseEnv->complexRelation << endl;
        }
;

var_arg: VAR
        {
            int vn =
                Cor_varname_enter(&(Cor_parserStruct->rule_var_names), $1);
            $$ = new Cor_CArg();
            $$->set_var(vn);
        }

member_list:    member_list COMMA arg_fact
        {
            Cor_insert_into_set($1, $3);
            $3->destroy();
        }

    |       arg_fact
        {

            $$ = Cor_create_nested_multiset();
            Cor_insert_into_set($$, $1);
            $1->destroy();
        }
    |       /* Nothing - allows the empty set {} */
        {
            $$ = Cor_create_nested_multiset();
        }
;

opt_adorn:    /* Nothing */
        {
            $$ = NULL;
        }
    |    LBRACKET id_list RBRACKET
        {
            $$ = $2;
        }
;

id_list:
         id id_list
    |    id
;

id:      symbol
    |    VAR
;

symbol:  SYMBOL
    |    FSYMBOL
;

%%

// *************************************************************************
void yyerror(char *s)
{
    *(Cor_exEnv->err_strm) << s << " line "
            << Cor_parseEnv->cur_line_num << ":\n"
            << Cor_parseEnv->commandbuf << endl;

    if (Cor_exEnv->C_interrupt_raised > 0) {

        *(Cor_exEnv->err_strm)
               << "CORAL:: Interrupt occurred during execution!:";
        *(Cor_exEnv->err_strm) << "resetting state !\n";
        *(Cor_exEnv->err_strm) << "Type a period to continue.\n" << flush;

          // abort a consult that is halfway thru
        Cor_parseStack->abort_file();
        Cor_parserStruct->reset(1);
        Cor_exEnv->C_interrupt_raised = 0;
    }
}

// *************************************************************************
