// *************************************************************************
//    Schema.C
// *************************************************************************

#ifdef WITH_PERSISTENCE


#include "Globals.h"
#include "CoralError.h"
#include "SMUtil.h"
#include "sm_client.h"
#include "Config.h"
#include "Schema.h"
#include "IndexStruct.h"
#include "BPlusTreeIndex.h"
#include "FileDescStruct.h"
#include "BufferGroup.h"
#include "Transaction.h"
#include "Catalog.h"
#include "StorageManager.h"
#include "Debug.h"

// *************************************************************************
Cor_Schema::Cor_Schema(Cor_ArgList &arglist, int& error, int by_example)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    int i = 0;

    if (by_example) {

          // Figure out the schema from a sample fact.

        FOR_EACH_ARG(arg, arglist) {

            if (arg->Kind() == INT_ARG) {
                types[i] = COR_INT_DBTYPE;

            } else if (arg->Kind() == DOUBLE_ARG) {
                types[i] = COR_DOUBLE_DBTYPE;
    
            } else if (arg->Kind() == SYMBOL_ARG) {
                types[i] = COR_STRING_DBTYPE;
    
            } else {
                error = COR_BAD_SCHEMA;
                return;
            }

            i++;
    
        } END_EACH_ARG

    } else {

        FOR_EACH_ARG(arg, arglist) {
    
            if (arg->equals("int")) {
                types[i] = COR_INT_DBTYPE;
    
            } else if (arg->equals("double")) {
                types[i] = COR_DOUBLE_DBTYPE;
    
            } else if (arg->equals("string")) {
                types[i] = COR_STRING_DBTYPE;
    
            } else {
                error = COR_BAD_SCHEMA;
                return;
            }
        
            i++;
    
        } END_EACH_ARG
    }

    arity = i;
}

// *************************************************************************
void Cor_Schema::operator=(Cor_Schema& schema2)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    for (int i = 0; i < COR_MAX_ARGLIST_SIZE; i++ )
        types[i] = schema2.types[i];
    arity = schema2.arity;
}

// *************************************************************************
void Cor_Schema::printon(ostream *outf)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    *outf << "(";
    for (int i = 0; i < arity; i++) {
        switch (types[i]) {
        case COR_INT_DBTYPE:
            *outf << "int";
            break;
        case COR_DOUBLE_DBTYPE:
            *outf << "double";
            break;
        case COR_STRING_DBTYPE:
            *outf << "string";
            break;
        default:
            break;
        }

        if (i < arity - 1)
            *outf << ", ";
    }
    *outf << ")";
}

// *************************************************************************

#endif WITH_PERSISTENCE
