// *************************************************************************
//    DBSolvers.C
// *************************************************************************

#include <string.h>
#include <limits.h>
#include <sys/types.h>
#include <unistd.h>

#include "DBSolvers.h"

#include "Globals.h"
#include "BuiltinRelation.h"
#include "BuiltinSymbols.h"
#include "CoralError.h"
#include "Util.h"
#include "Database.h"
#include "TupleIterator.h"
#include "ClientProtocol.h"
#include "CArg.h"

// *************************************************************************

        // For parsing from strings
#define BUFF_MAX 5000

// *************************************************************************
#define FAIL_RETURN {iterator.set_no_match(); return(iterator.bindenv);}

Cor_BindEnv *Cor_CreateDBSolver(Cor_BuiltinRelation& ,
        Cor_TupleIterator& iterator)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    if (iterator.arg_list.count() != 1) {
        CORAL_error(COR_BAD_ARITY, "One argument to ws_create needed", NULL);
        FAIL_RETURN
    }
    else {
        Cor_Term term1 (iterator.arg_list[0], iterator.bindenv);
        Cor_deep_dereference_term (term1);
        if (!term1.expr.is_symbol()) {
            CORAL_error(COR_BAD_ARG_TYPE, NULL, "CreateDBSolver");
            term1.printon(Cor_exEnv->err_strm);
            *(Cor_exEnv->err_strm) << endl;
            FAIL_RETURN
        }
        
        char *actual_string = term1.expr.symbol();
        Cor_CArg act_arg = Cor_CArg(actual_string);
        Cor_createDB(act_arg);
    }
    return(iterator.bindenv);        
}

#undef FAIL_RETURN

// *************************************************************************
#define FAIL_RETURN {iterator.set_no_match(); return(iterator.bindenv);}

Cor_BindEnv *Cor_DestroyDBSolver(Cor_BuiltinRelation&,
            Cor_TupleIterator& iterator)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    if (iterator.arg_list.count() != 1) {
        CORAL_error(COR_BAD_ARITY, "One argument to ws_destroy needed", NULL);
        FAIL_RETURN
    } else {
        Cor_Term term1 (iterator.arg_list[0], iterator.bindenv);
        Cor_deep_dereference_term (term1);
        if (!term1.expr.is_symbol() ) {
            CORAL_error(COR_BAD_ARG_TYPE, NULL, "DestroyDBSolver");
            term1.printon(Cor_exEnv->err_strm);
            *(Cor_exEnv->err_strm) << endl;
            FAIL_RETURN
        }

        char *actual_string = term1.expr.symbol();
        Cor_CArg act_arg(actual_string);
        Cor_destroyDB(act_arg);
    }
    return(iterator.bindenv);        
}

#undef FAIL_RETURN

// *************************************************************************
#define FAIL_RETURN do { iterator.set_no_match(); \
                         return(iterator.bindenv); } while(0)

Cor_BindEnv *Cor_CurrentDBSolver(Cor_BuiltinRelation&,
        Cor_TupleIterator& iterator)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    if (iterator.arg_list.count() == 0) {
        *(Cor_exEnv->out_strm) << "Current Workspace is: "
                << Cor_CurDB->name.symbol() << endl;
        if (coralConfig->server_mode){
              // Modified to use Client protocol
            *(Cor_exEnv->out_strm) << CP_NEW_WS
                    << Cor_CurDB->name.symbol() << endl << flush;
        }
        return(iterator.bindenv);
    }

    if (iterator.arg_list.count() != 1) {
        CORAL_error(COR_BAD_ARITY, "One argument to ws_current needed", NULL);
        FAIL_RETURN;
    } else {
        Cor_Term term1 (iterator.arg_list[0], iterator.bindenv);
        Cor_deep_dereference_term (term1);

        if (term1.expr.is_var()) {
            Cor_CArg wsarg = Cor_CArg(Cor_CurDB->name.symbol());
            Cor_Term  wsterm = Cor_Term(&wsarg);
            iterator.bindenv->bind(term1.expr.var_num(), wsterm);
            if (coralConfig->server_mode){
                  // Modified to use Client protocol
                *(Cor_exEnv->out_strm) << CP_NEW_WS
                        << Cor_CurDB->name.symbol() << endl << flush;
            }
            return(iterator.bindenv);
        }

        if (!term1.expr.is_symbol() ) {
            CORAL_error(COR_BAD_ARG_TYPE, NULL, "CurrentDBSolver");
            term1.printon(Cor_exEnv->err_strm);
            *(Cor_exEnv->err_strm) << endl;
            FAIL_RETURN;
        }

        char *actual_string = term1.expr.symbol();
        Cor_CArg act_arg = Cor_CArg(actual_string);
        Cor_currentDB(act_arg);
        if (Cor_CurDB->name.equals("default_ws")) 
            Cor_parseEnv->prompt = coralConfig->prompt;
        else {
            strcpy(Cor_parseEnv->prompt_name,
                    Cor_CurDB->name.symbol());
            strcat(Cor_parseEnv->prompt_name, coralConfig->prompt);
        }

        if (coralConfig->server_mode){
              // Modified to use Client protocol
            *(Cor_exEnv->out_strm) << CP_NEW_WS
                    << Cor_CurDB->name.symbol() << endl << flush;
        }

      //  if (coralConfig->server_mode)
      //    *(Cor_exEnv->out_strm)
      //        << endl << CP_PRMT << Cor_parseEnv->prompt << endl;

          // Update Current database for the current user.
        Cor_curUser->Cor_CurDB = Cor_CurDB;

    }
    return(iterator.bindenv);                
}

#undef FAIL_RETURN

// *************************************************************************
#define FAIL_RETURN {iterator.set_no_match(); return(iterator.bindenv);}

Cor_BindEnv * Cor_ListDBSolver(Cor_BuiltinRelation&, Cor_TupleIterator& iterator)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    if (iterator.arg_list.count() != 0) {
        CORAL_error(COR_BAD_ARITY, "No arguments to list_ws needed", NULL);
        FAIL_RETURN
    }

    Cor_listEveryDB();

    return(iterator.bindenv);
}

#undef FAIL_RETURN

// *************************************************************************
#define FAIL_RETURN {iterator.set_no_match(); return(iterator.bindenv);}

Cor_BindEnv *Cor_SaveDBSolver(Cor_BuiltinRelation&, Cor_TupleIterator& iterator)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    char *actual_string1, *actual_string2;

    if ((iterator.arg_list.count() > 2) || (iterator.arg_list.count() < 1)) {
        CORAL_error(COR_BAD_ARITY, "Two arguments to ws_save needed", NULL);
        FAIL_RETURN
    }

    Cor_Term term1 (iterator.arg_list[0], iterator.bindenv);
    Cor_deep_dereference_term (term1);
    if (!term1.expr.is_symbol()) {
        CORAL_error(COR_BAD_ARG_TYPE, "should be a symbol", "SaveDBSolver");
        term1.printon(Cor_exEnv->err_strm);
        *(Cor_exEnv->err_strm) << endl;
        FAIL_RETURN
    }

    if (iterator.arg_list.count() == 2) {
        Cor_Term term2 (iterator.arg_list[1], iterator.bindenv);
        Cor_deep_dereference_term (term2);
        if (!term2.expr.is_symbol()) {
            CORAL_error(COR_BAD_ARG_TYPE, "should be a symbol", "SaveDBSolver");
            term2.printon(Cor_exEnv->err_strm);
            *(Cor_exEnv->err_strm) << endl;
            FAIL_RETURN;
        }
        actual_string2 = term2.expr.symbol();
    } else
        actual_string2 = NULL;

    actual_string1 = term1.expr.symbol();
    Cor_CArg act_arg2 = Cor_CArg(actual_string2);
    Cor_CArg no_arg = Cor_CArg();

    ostream *fd;
    if (!(fd = Cor_open_outfile(actual_string1,ios::app))) {
        CORAL_error(COR_CANNOT_OPEN_FILE, actual_string1, "SaveDBSolver");
        FAIL_RETURN        
    }

    Cor_saveDB((actual_string2 ? act_arg2 : no_arg), fd);
                
    delete fd;

    return(iterator.bindenv);                    
}

#undef FAIL_RETURN

// *************************************************************************
#define FAIL_RETURN {iterator.set_no_match(); return(iterator.bindenv);}

Cor_BindEnv *Cor_RenameDBSolver(Cor_BuiltinRelation&,
        Cor_TupleIterator& iterator)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    char *actual_string1, *actual_string2;

    if ((iterator.arg_list.count() < 1) || (iterator.arg_list.count() > 2)){
        CORAL_error(COR_BAD_ARITY, "Two arguments to ws_rename needed", NULL);
        FAIL_RETURN
    }

    Cor_Term term1 (iterator.arg_list[0], iterator.bindenv);
    Cor_deep_dereference_term (term1);
    if (!term1.expr.is_symbol()) {
        CORAL_error(COR_BAD_ARG_TYPE, "should be a symbol", "RenameDBSolver");
        term1.printon(Cor_exEnv->err_strm);
        *(Cor_exEnv->err_strm) << endl;;
        FAIL_RETURN
    }

    if (iterator.arg_list.count() == 1) {
        actual_string1 = term1.expr.symbol();
        Cor_CArg act_arg1 = Cor_CArg(actual_string1);
        Cor_renameDB(Cor_CurDB->name, act_arg1);
        return(iterator.bindenv);        
    }

    Cor_Term term2 (iterator.arg_list[1], iterator.bindenv);
    Cor_deep_dereference_term (term2);
    if (!term2.expr.is_symbol()) {
        CORAL_error(COR_BAD_ARG_TYPE, "should be a symbol", "RenameDBSolver");
        term2.printon(Cor_exEnv->err_strm);
        *(Cor_exEnv->err_strm) << endl;;
        FAIL_RETURN
    }
    actual_string1 = term1.expr.symbol();
    actual_string2 = term2.expr.symbol();
    Cor_CArg act_arg1 = Cor_CArg(actual_string1);
    Cor_CArg act_arg2 = Cor_CArg(actual_string2);

    Cor_renameDB(act_arg1, act_arg2);
    
    return(iterator.bindenv);        
}

#undef FAIL_RETURN

// *************************************************************************
#define FAIL_RETURN {iterator.set_no_match(); return(iterator.bindenv);}

Cor_BindEnv *Cor_OpenRelSolver(Cor_BuiltinRelation&,
        Cor_TupleIterator& iterator)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    if (iterator.arg_list.count() != 2) {
        CORAL_error(COR_BAD_ARITY, "Two arguments to rel_open needed", NULL);
        FAIL_RETURN
    }

    Cor_Term term1 (iterator.arg_list[0], iterator.bindenv);
    Cor_deep_dereference_term(term1);
    if (!term1.expr.is_functor()) {
        CORAL_error(COR_BAD_ARG_TYPE, "should be a functor", "OpenRelSolver");
        term1.printon(Cor_exEnv->err_strm);
        *(Cor_exEnv->err_strm) << endl;
        FAIL_RETURN
    }

    Cor_Term term2 (iterator.arg_list[1], iterator.bindenv);
    Cor_deep_dereference_term(term2);
    if (!term2.expr.is_symbol()) {
        CORAL_error(COR_BAD_ARG_TYPE, "should be a symbol", "OpenRelSolver");
        term2.printon(Cor_exEnv->err_strm);
        *(Cor_exEnv->err_strm) << endl;
        FAIL_RETURN
    }

    char *actual_string2 = term2.expr.symbol();
    Cor_CArg act_arg2 = Cor_CArg(actual_string2);

    Cor_openRelInCurDB(&(term1.expr), act_arg2);
    
    return(iterator.bindenv);        
}

#undef FAIL_RETURN

// *************************************************************************
#define FAIL_RETURN {iterator.set_no_match(); return(iterator.bindenv);}

Cor_BindEnv *Cor_RelDisplay(Cor_BuiltinRelation&, Cor_TupleIterator& iterator)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    char name[100];
    char *tmp_file_name = Cor_mk_tmp_file_name(name);
    ostream *tmp_file = Cor_open_file(tmp_file_name);
    if (!tmp_file) {
        FAIL_RETURN
    }

    if (iterator.arg_list.count() == 0) {
        Cor_listRelsInDB(Cor_CurDB->name, tmp_file);
    }
    else if (iterator.arg_list.count() == 1) {
        Cor_Term term1 (iterator.arg_list[0], iterator.bindenv);
        Cor_deep_dereference_term (term1);
        if (!term1.expr.is_symbol()) {
            CORAL_error(COR_BAD_ARG_TYPE, "should be a symbol", "RelDisplay");
            term1.printon(Cor_exEnv->err_strm);
            *(Cor_exEnv->err_strm) << endl;
            FAIL_RETURN
        }
        char *actual_string = term1.expr.symbol();
        Cor_CArg act_arg = Cor_CArg(actual_string);
        Cor_listRelsInDB(act_arg, tmp_file);
    } else {
        delete tmp_file;
        CORAL_error(COR_BAD_ARITY,
            "Zero or one argument to list_rels needed", NULL);
        FAIL_RETURN
    }

    delete tmp_file;

    char name1[100];
    char *tmp_file_name1 = Cor_mk_tmp_file_name(name1);

    char buf[200];
    sprintf(buf, "sort +0 +0 %s > %s", tmp_file_name, tmp_file_name1);
    if (system(buf)) {
        Cor_DisplayFile(tmp_file_name);
    }
    else {
        Cor_DisplayFile(tmp_file_name1);
        unlink(tmp_file_name1);
    }

    unlink(tmp_file_name);
    return(iterator.bindenv);
}

#undef FAIL_RETURN

// *************************************************************************
#define FAIL_RETURN {iterator.set_no_match(); return(iterator.bindenv);}

Cor_BindEnv *Cor_CloseRelSolver(Cor_BuiltinRelation&,
            Cor_TupleIterator& iterator)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    if ((iterator.arg_list.count() != 2) && (iterator.arg_list.count() != 1)) {
        CORAL_error(COR_BAD_ARITY, "One or two arguments to rel_open needed", NULL);
        FAIL_RETURN
    }

    Cor_Term term1 (iterator.arg_list[0], iterator.bindenv);
    Cor_deep_dereference_term (term1);
    if (!term1.expr.is_functor()) {
        CORAL_error(COR_BAD_ARG_TYPE, "should be a func-arg", "CloseRelSolver");
        term1.printon(Cor_exEnv->err_strm);
        *(Cor_exEnv->err_strm) << endl;
        FAIL_RETURN
    }

    if (iterator.arg_list.count() == 1) {
        Cor_removeRelFromDB(&(term1.expr), Cor_CurDB->name);
        return(iterator.bindenv);
    }

    Cor_Term term2 (iterator.arg_list[1], iterator.bindenv);
    Cor_deep_dereference_term (term2);

    if (!term2.expr.is_symbol()) {
        CORAL_error(COR_BAD_ARG_TYPE, "should be a symbol", "CloseRelSolver");
        term2.printon(Cor_exEnv->err_strm);
        *(Cor_exEnv->err_strm) << endl;
        FAIL_RETURN
    }

    char *actual_string2 = term2.expr.symbol();
    Cor_CArg act_arg2 = Cor_CArg(actual_string2);
    Cor_removeRelFromDB(&(term1.expr), act_arg2);

    return(iterator.bindenv);        
}

#undef FAIL_RETURN

// *************************************************************************
#define FAIL_RETURN {iterator.set_no_match(); return(iterator.bindenv);}

Cor_BindEnv *Cor_ClearRelSolver(Cor_BuiltinRelation&,
        Cor_TupleIterator& iterator)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    if ((iterator.arg_list.count() != 2) && (iterator.arg_list.count() != 1)) {
        CORAL_error(COR_BAD_ARITY, 
                "One or two arguments to rel_clear needed", NULL);
        FAIL_RETURN
    }

    Cor_Term term1 (iterator.arg_list[0], iterator.bindenv);
    Cor_deep_dereference_term (term1);
    if (!term1.expr.is_functor()) {
        CORAL_error(COR_BAD_ARG_TYPE, "should be a func-arg", "ClearRelSolver");
        term1.printon(Cor_exEnv->err_strm);
        *(Cor_exEnv->err_strm) << endl;
        FAIL_RETURN
    }

    if (iterator.arg_list.count() == 1) {
        Cor_clearRelInDB(&(term1.expr), Cor_CurDB->name);
        return(iterator.bindenv);
    }

    Cor_Term term2 (iterator.arg_list[1], iterator.bindenv);
    Cor_deep_dereference_term (term2);

    if (!term2.expr.is_symbol()) {
        CORAL_error(COR_BAD_ARG_TYPE, "should be a symbol", "ClearRelSolver");
        term2.printon(Cor_exEnv->err_strm);
        *(Cor_exEnv->err_strm) << endl;
        FAIL_RETURN
    }

    char *actual_string2 = term2.expr.symbol();
    Cor_CArg act_arg2 = Cor_CArg(actual_string2);
    Cor_clearRelInDB(&(term1.expr), act_arg2);

    return(iterator.bindenv);        
}

#undef FAIL_RETURN

// *************************************************************************
#define FAIL_RETURN {iterator.set_no_match(); return(iterator.bindenv);}

Cor_BindEnv *Cor_CopyRelSolver(Cor_BuiltinRelation&,
        Cor_TupleIterator& iterator)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    if ((iterator.arg_list.count() != 2) && (iterator.arg_list.count() != 3)) {
        CORAL_error(COR_BAD_ARITY,
            "Two or three arguments to rel_add needed", NULL);
        FAIL_RETURN
    }

    Cor_Term term1 (iterator.arg_list[0], iterator.bindenv);
    Cor_deep_dereference_term (term1);
    if (!term1.expr.is_functor()) {
        CORAL_error(COR_BAD_ARG_TYPE, "should be a relation template",
        "CopyRelSolver");
        term1.printon(Cor_exEnv->err_strm);
        *(Cor_exEnv->err_strm) << endl;
        FAIL_RETURN
    }

    Cor_Term term2 (iterator.arg_list[1], iterator.bindenv);
    Cor_deep_dereference_term (term2);
    if (!term2.expr.is_symbol()) {
        CORAL_error(COR_BAD_ARG_TYPE, "should be a symbol", "CopyRelSolver");
        term2.printon(Cor_exEnv->err_strm);
        *(Cor_exEnv->err_strm) << endl;
        FAIL_RETURN
    }

    char *actual_string2 = term2.expr.symbol();
    Cor_CArg act_arg2 = Cor_CArg(actual_string2);

    if (iterator.arg_list.count() == 2) {
        Cor_copyToDB(&(term1.expr), act_arg2, &(term1.expr));
        return(iterator.bindenv);
    }

    Cor_Term term3 (iterator.arg_list[2], iterator.bindenv);
    Cor_deep_dereference_term (term3);
    if (!term3.expr.is_functor()) {
        CORAL_error(COR_BAD_ARG_TYPE, "should be a relation template",
            "CopyRelSolver");
        term3.printon(Cor_exEnv->err_strm);
        *(Cor_exEnv->err_strm) << endl;
        FAIL_RETURN
    }

    Cor_copyToDB(&(term1.expr), act_arg2, &(term3.expr));

    return(iterator.bindenv);
}

#undef FAIL_RETURN

// *************************************************************************
#define FAIL_RETURN {iterator.set_no_match(); return(iterator.bindenv);}

Cor_BindEnv *Cor_RenameRelSolver(Cor_BuiltinRelation&,
        Cor_TupleIterator& iterator)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    char *actual_string1, *actual_string2;

    if ((iterator.arg_list.count() < 2) || (iterator.arg_list.count() > 2)){
        CORAL_error(COR_BAD_ARITY, "Two arguments to rel_rename needed", NULL);
        FAIL_RETURN
    }

    Cor_Term term1 (iterator.arg_list[0], iterator.bindenv);
    Cor_deep_dereference_term (term1);
    if (!term1.expr.is_symbol()) {
        CORAL_error(COR_BAD_ARG_TYPE, "should be a symbol", "RenameRelSolver");
        term1.printon(Cor_exEnv->err_strm);
        *(Cor_exEnv->err_strm) << endl;;
        FAIL_RETURN
    }

    Cor_Term term2 (iterator.arg_list[1], iterator.bindenv);
    Cor_deep_dereference_term (term2);

    if (!term2.expr.is_symbol()) {
        CORAL_error(COR_BAD_ARG_TYPE, "should be a symbol", "RenameRelSolver");
        term2.printon(Cor_exEnv->err_strm);
        *(Cor_exEnv->err_strm) << endl;;
        FAIL_RETURN
    }

    actual_string1 = term1.expr.symbol();
    actual_string2 = term2.expr.symbol();
    Cor_CArg act_arg1 = Cor_CArg(actual_string1);
    Cor_CArg act_arg2 = Cor_CArg(actual_string2);

    Cor_renameRelation(act_arg1, act_arg2);

    return(iterator.bindenv);
}

#undef FAIL_RETURN

// *************************************************************************
#define FAIL_RETURN {iterator.set_no_match(); return(iterator.bindenv);}

Cor_BindEnv *Cor_AddRelSolver(Cor_BuiltinRelation&, Cor_TupleIterator& iterator)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    if ((iterator.arg_list.count() != 2) && (iterator.arg_list.count() != 3)) {
        CORAL_error(COR_BAD_ARITY,
            "Two or three arguments to rel_add needed", NULL);
        FAIL_RETURN
    }

    Cor_Term term1 (iterator.arg_list[0], iterator.bindenv);
    Cor_deep_dereference_term (term1);
    if (!term1.expr.is_functor()) {
        CORAL_error(COR_BAD_ARG_TYPE, "should be a relation template",
            "AddRelSolver");
        term1.printon(Cor_exEnv->err_strm);
        *(Cor_exEnv->err_strm) << endl;
        FAIL_RETURN
    }

    Cor_Term term2 (iterator.arg_list[1], iterator.bindenv);
    Cor_deep_dereference_term (term2);
    if (!term2.expr.is_symbol()) {
        CORAL_error(COR_BAD_ARG_TYPE, "should be a symbol", "AddRelSolver");
        term2.printon(Cor_exEnv->err_strm);
        *(Cor_exEnv->err_strm) << endl;
        FAIL_RETURN
    }

    char *actual_string2 = term2.expr.symbol();
    Cor_CArg act_arg2 = Cor_CArg(actual_string2);

    if (iterator.arg_list.count() == 2) {
        Cor_addToDB(&(term1.expr), act_arg2, &(term1.expr));
        return(iterator.bindenv);
    }

    Cor_Term term3 (iterator.arg_list[2], iterator.bindenv);
    Cor_deep_dereference_term (term3);
    if (!term3.expr.is_functor()) {
        CORAL_error(COR_BAD_ARG_TYPE, "should be a relation template",
            "CloseRelSolver");
        term3.printon(Cor_exEnv->err_strm);
        *(Cor_exEnv->err_strm) << endl;
        FAIL_RETURN
    }

    Cor_addToDB(&(term1.expr), act_arg2, &(term3.expr));

    return(iterator.bindenv);        
}

#undef FAIL_RETURN

// *************************************************************************
// This is an example of how to 'macro' a number of builtins under
// a single name -- PRAVEEN. More work needs to be done on this
// *************************************************************************

Cor_BindEnv *Cor_ListAllSolver(Cor_BuiltinRelation&, Cor_TupleIterator& iterator)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    char buf[BUFF_MAX];

    sprintf(buf, "list_dbs. list_rels.");
    Cor_parseStack->handle_new_buff(buf,"buffer");
    Cor_parseStack->parse(Cor_exEnv->err_strm,1,"Coral Internal Error.\n");

    return(iterator.bindenv);
}

// *************************************************************************
#define FAIL_RETURN {iterator.set_no_match(); return(iterator.bindenv);}

Cor_BindEnv *Cor_ClearDBSolver(Cor_BuiltinRelation&,
        Cor_TupleIterator& iterator)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    char buf[BUFF_MAX];
    Cor_CArg name;

    if (iterator.arg_list.count() > 1) {
        CORAL_error(COR_BAD_ARITY, "One argument to ws_clear needed", NULL);
        FAIL_RETURN
    }

    if (iterator.arg_list.count() == 0)
        name = Cor_CurDB->name;
    else {
        Cor_Term term1 (iterator.arg_list[0], iterator.bindenv);
        Cor_deep_dereference_term (term1);
        if (!term1.expr.is_symbol()) {
            CORAL_error(COR_BAD_ARG_TYPE, NULL, "ClearDBSolver");
            term1.printon(Cor_exEnv->err_strm);
            *(Cor_exEnv->err_strm) << endl;
            FAIL_RETURN
        }
        name = term1.expr;
    }

    sprintf(buf, "%s%s %s%s %s%s%s%s %s%s%s%s %s%s%s%s",
        Cor_CreateDBSymbol->symbol(),"(\"__tmp\").\n",
        Cor_CurrentDBSymbol->symbol(),"(\"__tmp\").\n",
        Cor_DestroyDBSymbol->symbol(), "(", name.symbol() , ").\n",
        Cor_RenameDBSymbol->symbol(),"(\"__tmp\",",
                name.symbol(), ").\n",
        Cor_CurrentDBSymbol->symbol(),
                "(",Cor_CurDB->name.symbol(),").\n");

    Cor_parseStack->handle_new_buff(buf,"buffer");
    Cor_parseStack->parse(Cor_exEnv->err_strm,1,"Coral Internal Error.\n");

    return(iterator.bindenv);
}

#undef FAIL_RETURN

// *************************************************************************
