// *************************************************************************
//    Tuple.C
// *************************************************************************

#include "Tuple.h"
#include "BindEnv.h"
#include "Globals.h"
#include "Debug.h"

// *************************************************************************
/*
Cor_Tuple::~Cor_Tuple()
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
}
*/

// *************************************************************************
int Cor_Tuple::delete_self(int force_del)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    // Do the reference counting for deleted tuples

    if (force_del || ( is_deleted() && (decr_del_ref() == 0)) ) {
        if (! has_no_args()) {
            delete _args;
        }
        delete this;
        return(1);
    }
    return(0);
}

// *************************************************************************
Cor_CArg& Cor_Tuple::operator [](int i) const
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    // This is no longer needed since Tuple's have no BindEnv,
    // the new way allows tuples to be modified as well. -SPF

    //Cor_Term *term1 = term(i);
    
    //Cor_full_dereference_term(*term1);
    //return term1->expr;

    return( (*_args)[i] );
}

// *************************************************************************
int Cor_Tuple::operator ==(const Cor_Tuple& that) const
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    // Compares only fully instantiated tuples (ignores bindings)

    for(int i=0;i < arity();i++)
        if ( ! (args()[i] == that.args()[i]) )
            return(0);

    return(1);
}

// *************************************************************************
int Cor_Tuple::isConstant()
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    for(int i=0;i < arity();i++)
        if (! arg(i).isConstant())
            return(0); 

    return(1);
}

// *************************************************************************
// Check for equality
int Cor_Tuple::equals(Cor_Tuple *that)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    int j, n;

    if (this == that)
        return(1);

    if (!this || !that)
        return(0);

    n = _args->arity();

    if (n != that->_args->arity())
        return(0);

    for (j=0; j < n; j++)
        if ( ! (*_args)[j].equals( (*(that->_args))[j] ) )
            return(0);

    return(1);
}

// *************************************************************************
// Instantiate all variables that are bound in this bindenv.
void Cor_Tuple::instantiate( Cor_BindEnv *env )
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    if (env) {
        FOR_EACH_ARG(arg, *_args) {
            arg->simplify_me(env);
        } END_EACH_ARG
    }
}

// *************************************************************************
void Cor_Tuple::sprint(char *buff, int *pos) const
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    _args->sprint(buff, pos, NULL);
}

// *************************************************************************
void Cor_Tuple::printon(ostream *file) const
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    _args->print(NULL, file);
}

// *************************************************************************
ostream& operator<< (ostream& out, const Cor_Tuple& tuple)
{
    tuple._args->print(NULL,&out);
    return(out);
}

// *************************************************************************
