// *************************************************************************
//    TupleStack.h
// *************************************************************************

#ifndef _TupleStack_H_
#define _TupleStack_H_

#include <iostream.h>
#include <stdlib.h>

#include "HashVal.h"
#include "Tuple.h"
#include "Entity.h"

// *************************************************************************

class Cor_TupleStack {

  private:
    unsigned int   length;     // The amount of the array being used
    unsigned int   array_size; // size of the arrays that were newed

  public:
    HashVal    *hashs;           // Array of hash values
    Cor_Tuple **tuples;          // Array of tuples

#ifdef    REUSE_MEMORY
    void *operator new(size_t size) {
        return(Coral_Free->acquire(size));
    }

    void  operator delete(void *trash, size_t size) {
        Coral_Free->dispose((Cor_Ent_Element*) trash, size);
    }
#endif


    Cor_TupleStack() {
        hashs = NULL;
        tuples = NULL;
        length = 0;
        array_size = 0;
    }

    Cor_TupleStack(int sze) {
        hashs = new HashVal [sze];
        tuples = new Cor_Tuple* [sze];
        length = 0;
        array_size = sze;
    }

   ~Cor_TupleStack() {
                        delete [] hashs;
                        delete [] tuples;
                     }

    void free_all(int deleteTuples=1);

    int  count() { return(length); }
    int  is_empty() { return(length == 0); }

    int  check_if_empty() {
        for (unsigned int j=0; j < length; j++)
            if (tuples[j])
                return(0);
        empty();
        return(1);        
    }

      // push() returns the position
    int  push(HashVal hash, Cor_Tuple *tpl);

    Cor_Tuple *get_ith_tuple(unsigned int i) {
        if (i < length)
            return(tuples[i]);
        return(NULL);
    }

    HashVal get_ith_hash(unsigned int i) {
        if (i < length)
            return(hashs[i]);
        return(UnknownHashValue);
    }

    void clr_ith(unsigned int i) {
        if (i < length) {
            if (i == length-1)
                length--;
            tuples[i] = NULL;
        }
    }

    void set_ith(unsigned int i, HashVal hash, Cor_Tuple *tpl) {
        if (i < length) {
            hashs[i]  = hash;
            tuples[i] = tpl;
        }
    }

    int delete_tuple(Cor_Tuple *tpl) {
        for (unsigned int j=0; j < length; j++) {
            if (tuples[j] == tpl) {
                clr_ith(j);
                return(1);
            }
        }
        return(0);
    }

    int delete_eq_tuple(Cor_Tuple *tpl) {
        for (unsigned int j=0; j < length; j++) {
            if (tuples[j]) {
                if (*(tuples[j]) == *tpl) {
                    delete tuples[j];
                    clr_ith(j);
                    return(1);
                }
            }
        }
        return(0);
    }

    void copy(Cor_TupleStack&);

      // Empty out the stack
    void empty() { length = 0; }


      // Compress this stack if possible, returning the translated
      // position of the input position
    int  compress(int pos);

    friend ostream& operator<< (ostream& out, const Cor_TupleStack& tstk);
};

// *************************************************************************

#endif _TupleStack_H_
