// *************************************************************************
//    Main.C
// *************************************************************************

#include <stdio.h>
#include <string.h>
#include <libc.h>

#include "HandleExport.h"
#include "Translate.h"
#include "OutputExport.h"
#include "Macro.h"
#include "Lexer.h"
#include "Stack.h"

/*****************************************************************************/

char *input_file_name = NULL;
FILE *input_file;
char *output_file_name = NULL;
FILE *output_file;
char *error_msgs_file = NULL;
FILE *error_file;
char *coral_f = NULL;

StringStack *save_exlists = new StringStack();

/*****************************************************************************/
void open_input_file(char *filename)
{

    if (filename && ((input_file = fopen(filename,"r")) != NULL)) {
        input_file_name = strdup(filename);

    } else {
        if (filename){
            fprintf(stderr,"Cannot open file: %s\n", filename);
            exit(1);
        }
        input_file = stdin;
        input_file_name = strdup("stdin");
    }

    ttin = input_file;
}

/*****************************************************************************/
void open_output_file(char *filename) 
{
    if (output_file_name)
        fputs("Warning : an output file may already be open.\n",stderr);
   
    if (filename && ((output_file = fopen(filename, "w")) != NULL)) {
        output_file_name = strdup(filename);

    } else {
        if (filename)
            fprintf(stderr,"Cannot open file: %s\n", filename);
        output_file = stdout;
        output_file_name = strdup("stdout");
    }

    ttout = output_file;
}

/*****************************************************************************/
int main(int argc, char *argv[]) 
{
    int i = 1;
    char *in_file = NULL, *out_file = NULL, *err_file = NULL;
    int consults = 0;
  
    /*
        Currently translator is executed as
            translator -i inputfile -o outputfile -e errorfile -c coralfile
        the -c option gives the file where the CORAL code should be dumped
        when the resulting program is compiled. 
    */
    
    while (i < argc) {
        if (argv[i][0] == '-')    {
            switch (argv[i][1]) {
            case 'i':
                in_file = argv[++i];    
                break;
            case 'o':
                if (strcmp(argv[i], "-options") == 0) {
                    fprintf(stderr, "Usage is:\n");
                    fprintf(stderr, "%s -i inputfile -o outputfile",argv[0]);
                    fprintf(stderr, " -e errorfile -c coralfile\n");
                    fprintf(stderr, "Any option(s) can be skipped \n");
                    exit(1);
                }
                out_file = argv[++i];    
                break;
            case 'e':
                err_file = argv[++i];    
                break;
            case 'c' :
                coral_f = argv[++i];    
                break;
            default:
                fprintf(stderr, "unknown option %s \n", argv[i]); 
                fprintf(stderr, "Usage is \n");
                fprintf(stderr, "%s -i inputfile -o outputfile",argv[0]);
                fprintf(stderr, "  -e errorfile -c coralfile\n");
                fprintf(stderr, "Any option(s) can be skipped \n");
                exit(1);
            }
        }
        else {
            fprintf(stderr, "unknown argument %s \n", argv[i]); 
        }
        i++;
    }
        
    open_input_file(in_file);
    open_output_file(out_file);

    if (!coral_f) {
        coral_f = "/tmp/translator.out.P";
    }

    init(err_file, coral_f);
  
    //int current_position;
    //int end_position;
    //int line_no;
    
    Token tokn; 
    int tok_type;

    while((tok_type = ttlex(&tokn))){

        switch(tok_type){

        case my_CORAL_BEGIN: {
            infoStruct info;

            consults = 0;
            fputs("\n\n  // To handle CORAL code\n",ttout);
            fputs("  temp__inits();\n",ttout);

            char buf[256];
            char *temp_str = new_name();
            save_exlists->push(temp_str);

            fprintf(ttout, "  extern Cor_ExecutionList *%s;\n", temp_str);
            sprintf(buf,
              "static Cor_ExecutionList *%s;\n", temp_str);
            exec_array_decl->append(buf);

            // Rodin needs this section?
            fprintf(ttout, "  if (!(%s->initialized)) {\n", temp_str);
            fprintf(ttout, "    %s();\n", exec_array_init_fn_name);
            fputs("  }\n",ttout);

            consults = handle_coral_code(info);

            char *new_temp_str = new_name();

// *************************************************************************
            if (info.no_of_params) {

                char *new_carg_name = new_name();

//                fprintf(ttout, "  Cor_ParameterStruct %s[%d];\n",
//                           new_carg_name, info.no_of_params);
                fprintf(ttout, "  Cor_CArg *%s;\n", new_carg_name);
                fprintf(ttout, "  %s->params.empty();\n", temp_str);

                int i;
                for (i = info.no_of_params-1; i >= 0; i--) {

                    switch(info.type_array[i]) {

                    case STRING_T:
                        fprintf(ttout, "  %s = new Cor_CArg((char *) %s);\n",
                                new_carg_name, info.var_names[i] );
                        fprintf(ttout, "  %s->params.push(%s);\n",
                                temp_str, new_carg_name);
                        break;

                    case INT_T:
                        fprintf(ttout, "  %s = new Cor_CArg((int) %s);\n",
                                new_carg_name, info.var_names[i] );
                        fprintf(ttout, "  %s->params.push(%s);\n",
                                temp_str, new_carg_name);
                        break;

                    case LONG_INT:
                            // long isn't really supported by CArg
                        fprintf(ttout, "  %s = new Cor_CArg((int) %s);\n",
                                new_carg_name, info.var_names[i] );
                        fprintf(ttout, "  %s->params.push(%s);\n",
                                temp_str, new_carg_name);
                        break;

                    case SHORT_INT:
                            // short isn't really supported by CArg
                        fprintf(ttout, "  %s = new Cor_CArg((int) %s);\n",
                                new_carg_name, info.var_names[i] );
                        fprintf(ttout, "  %s->params.push(%s);\n",
                                temp_str, new_carg_name);
                        break;

                    case FLOAT_T:
                            // float isn't really supported by CArg
                        fprintf(ttout, "  %s = new Cor_CArg((double) %s);\n",
                                new_carg_name, info.var_names[i] );
                        fprintf(ttout, "  %s->params.push(%s);\n",
                                temp_str, new_carg_name);
                        break;

                    case DOUBLE_T:
                        fprintf(ttout, "  %s = new Cor_CArg((double) %s);\n",
                                new_carg_name, info.var_names[i] );
                        fprintf(ttout, "  %s->params.push(%s);\n",
                                temp_str, new_carg_name);
                        break;

                    case CARG_T:
                        fprintf(ttout, "  %s = %s;\n",
                                new_carg_name, info.var_names[i] );
                        fprintf(ttout, "  %s->params.push(%s);\n",
                                temp_str, new_carg_name);
                        break;

                    case VOID_T:
                    case CHUNK_T:
                        fprintf(ttout, "  %s = new Cor_CArg();\n",
                                new_carg_name);
                        fprintf(ttout, "  %s->set_chunk((void *) %s);\n",
                                new_carg_name, info.var_names[i] );
                        fprintf(ttout, "  %s->params.push(%s);\n",
                                temp_str, new_carg_name);
                        break;

                    default:
                        fprintf(stderr,"Type in switch is %d\n",
                                info.type_array[i]);
                        break;
                    }
                }
            }
// *************************************************************************

            sprintf(buf, "  %s(%s);\n", new_temp_str, temp_str);
            exec_array_init_calls->append(buf);

            //    Still to put in initialization code
            generate_initialization_code(new_temp_str, info);
            fprintf(ttout, "  %s->execute();\n", temp_str);
                /* Add Cor_processInputStack() call for each consult */
            int j;
            for (j=0; j < consults; j++){
                fprintf(ttout, "  Cor_processInputStack();\n");
            }
            line_directive(ttout);

            delete [] temp_str;
            break;
        }

        case my_INIT_CORAL:
            //    Whenever there is a main program, the user has to make a call
            //    to init_coral() which initializes the database with some
            //    relations.  The call to dummy fn which enters the relations
            //    declared through the export calls is placed immediately after
            //    init_coral().

            output_call_dummy(dummy_fn_name);
            break;

        case my_EXIT_CORAL:
             // Lexer already handled it.
            break;

        case my_EXPORT_CORAL: {
            export_header_decl = new mystring;
            handle_export *export_info = new handle_export();
            handle_export_declaration(export_info, export_header_decl);

            fputs("\n",ttout);
            output_export_code(export_info, export_dummy_fn,
                dummy_temp_name, export_header_decl);

            delete export_info;
            delete export_header_decl;
            break;
        }

        case SYMBOL:
            free(tokn.val.str);
            break;

        default: fprintf(stderr,"Unknown Token. (%d)\n", tok_type);
                 break;
        }
    }

    close(export_dummy_fn);

    fputs("\nstatic void temp__inits() {\n",ttout);
    int llen = save_exlists->count();
    char *exnme0;

    if (llen){
        exnme0 = save_exlists->get_ith(0);
        fprintf(ttout,"    if (!%s) {\n",exnme0);
    }

    for (int j=0; j < llen; j++) {
        char *exname = save_exlists->get_ith(j);
        // SPF
        /*  fprintf(ttout,"  extern Cor_ExecutionList *%s;\n", exname);  */
        fprintf(ttout,"      %s = new Cor_ExecutionList();\n", exname);
    }

    fprintf(ttout, "      %s();\n", exec_array_init_fn_name);

    if (llen)
        fputs("    }\n}\n\n", ttout);
    else
        fputs("}\n\n", ttout);

    fputs("static void temp__deletes() {\n",ttout);
    while (!save_exlists->is_empty()) {
        char *exname = save_exlists->pop();
        /*  fprintf(ttout,"  extern Cor_ExecutionList *%s;\n", exname);  */
        fprintf(ttout,"  delete %s;\n", exname);
        delete [] exname;
    }
    fputs("}\n\n", ttout);
   
    if (input_file_name){
        delete [] input_file_name;
        input_file_name = NULL;
        if (input_file)
            fclose(input_file);
    }

    if (output_file_name){
        delete [] output_file_name;
        output_file_name = NULL;
        if (output_file)
            fclose(output_file);
    }

    delete save_exlists;
    return(0);
}

/*****************************************************************************/

